﻿using System;
using System.Text.Json;


using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Net.Http;
using System.Text.Json.Serialization;

namespace JSON {
    class _01_Basics {
        public static async Task Main1(string[] args) {

            //
            // JsonDocument erstellen
            //

            Console.WriteLine("----------------------");
            Console.WriteLine("JsonDocument erstellen");
            Console.WriteLine("----------------------");

            string data = @" [ {""vorname"": ""Tom"", ""Nachname"": ""Schmidt"", ""alter"":21}, 
                               {""vorname"": ""Tina"", ""Nachname"": ""Schlau"", ""alter"":18, ""friend"": {""vorname"": ""Tom"", ""Nachname"": ""Schmidt"", ""alter"":21}}
                             ]";

            JsonDocument doc = JsonDocument.Parse(data);
            JsonElement root = doc.RootElement;

            var u1 = root[0];
            var u2 = root[1];
            Console.WriteLine(u1);
            Console.WriteLine(u2);

            Console.WriteLine(u1.GetProperty("vorname"));
            Console.WriteLine(u1.GetProperty("alter"));

            Console.WriteLine(u2.GetProperty("vorname"));
            Console.WriteLine(u2.GetProperty("alter"));


            //
            // Enumerate Array
            //

            Console.WriteLine("----------------------");
            Console.WriteLine("Json Enumerate Array");
            Console.WriteLine("----------------------");


            var users = root.EnumerateArray();

            while (users.MoveNext()) {
                var user = users.Current;
                System.Console.WriteLine(user);

                var props = user.EnumerateObject();

                while (props.MoveNext()) {
                    var prop = props.Current;
                    Console.WriteLine($"{prop.Name}: {prop.Value}");
                }
            }

            //
            // JSON Serialize
            //

            Console.WriteLine("--------------");
            Console.WriteLine("Json Serialize");
            Console.WriteLine("--------------");

            User user1 = new User("Tom", "Schmidt", new MyDate(2002, 09, 15));
            Console.WriteLine(user1);

            var options = new JsonSerializerOptions { WriteIndented = true };
            string jsonString = JsonSerializer.Serialize(user1, options);
            Console.WriteLine(jsonString);

            string fileName = @"../../user.json";
            File.WriteAllText(fileName, jsonString);


            /* .net7+ 
            //
            // JSON Serialize Async
            //

            Console.WriteLine("--------------");
            Console.WriteLine("Json Serialize");
            Console.WriteLine("--------------");

            User user2 = new User("Tina", "Schlau", new MyDate(2005, 06, 08));
            Console.WriteLine(user2);
            var stream = File.Create(fileName);
            await JsonSerializer.SerializeAsync(stream, user2);
            await stream.DisposeAsync();
            Console.WriteLine(File.ReadAllText(fileName));
            */

            //
            // JSON Deserialize
            //

            Console.WriteLine("--------------");
            Console.WriteLine("Json Deserialize");
            Console.WriteLine("--------------");

            jsonString = File.ReadAllText(fileName);

            var userResurrected = JsonSerializer.Deserialize<User>(jsonString);

            Console.WriteLine(userResurrected);


            //
            // JSON parse async

            Console.WriteLine("--------------");
            Console.WriteLine("Json parse async");
            Console.WriteLine("--------------");


            var httpClient = new HttpClient();

            var url = "https://raw.githubusercontent.com/dotnet/core/master/release-notes/releases-index.json";
            var ts = await httpClient.GetStreamAsync(url);

            var resp = await JsonDocument.ParseAsync(ts);

            var root1 = resp.RootElement.GetProperty("releases-index");

            var elems1 = root1.EnumerateArray();

            while (elems1.MoveNext()) {
                var node = elems1.Current;
                Console.WriteLine(node);
            }


            /* .net7+ required

            using System.Net.Http.Json;

            //
            // JSON parse async get to rest-api

            Console.WriteLine("--------------");
            Console.WriteLine("Json parse async get");
            Console.WriteLine("--------------");


            var client = new HttpClient();

            url = "http://webcode.me/users.json";
            var data2 = await client.GetFromJsonAsync<Users2>(url);

            if (data2 != null) {
                foreach (var user in data.users2) {
                    Console.WriteLine(user);
                }
            }

        class Users2 {
            public List<User2> users { get; set; } = new List<User2>();
        }

        class User2 {
            [JsonPropertyName("id")]
            public int Id { get; set; }

            [JsonPropertyName("first_name")]
            public string FirstName { get; set; } = string.Empty;

            [JsonPropertyName("last_name")]
            public string LastName { get; set; } = string.Empty;

            [JsonPropertyName("email")]
            public string Email { get; set; } = string.Empty;

            public override string ToString() {
                return $"User {{ {Id}| {FirstName} {LastName}| {Email} }}";
            }
        }



             */

            Console.WriteLine("Done.");



            Console.ReadKey();
        }

        class MyDate {
            public int year { get; set; }
            public int month { get; set; }
            public int day { get; set; }
            public MyDate(int year, int month, int day) {
                this.year = year;
                this.month = month;
                this.day = day;
            }
            public override string ToString() {
                return $"{year}/{month}/{day}";
            }
        }
        class User {
            public string vorname { get; set; }
            public string nachname { get; set; }
            public MyDate geburtstag { get; set; }
            public User(string vorname, string nachname, MyDate geburtstag) {
                this.vorname = vorname;
                this.nachname = nachname;
                this.geburtstag = geburtstag;
            }
            public override string ToString() {
                return $"{vorname},{ nachname},{ geburtstag}";
            }
        }
    }
}
